#######################################################
###  __init__.py
###  This is the fundamental python module.  I import 
###  everything into this namespace to shorten the 
###  calling tree when this module is used in other
###  scripts.  This also contains the version.
#######################################################

from subnetfinder.subnetfinder import *
__version__='1.2.1'